"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const request_promise_native_1 = __importDefault(require("request-promise-native"));
const framework_1 = require("../homebridge/framework");
class TionAuthApi {
    constructor(log, config, storage) {
        this.log = log;
        this.config = config;
        this.authStorage = storage;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.myAuthData = yield this.authStorage.load();
            }
            catch (err) {
                err = null; // relax lint
            }
        });
    }
    getAccessToken() {
        return this.myAuthData && this.myAuthData.access_token;
    }
    authenticateUsingPassword() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._internalAuthenticate({
                grant_type: 'password',
                username: this.config.userName,
                password: this.config.password,
                client_id: TionAuthApi.clientId,
                client_secret: TionAuthApi.clientSecret,
            });
            yield this.authStorage.save(this.myAuthData);
            return this.myAuthData && this.myAuthData.access_token;
        });
    }
    authenticateUsingRefreshToken() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._internalAuthenticate({
                grant_type: 'refresh_token',
                refresh_token: this.myAuthData.refresh_token,
                client_id: TionAuthApi.clientId,
                client_secret: TionAuthApi.clientSecret,
            });
            return this.myAuthData.access_token;
        });
    }
    _internalAuthenticate(params) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.myAuthData = yield request_promise_native_1.default.post(TionAuthApi.oAuthUrl, { form: params, json: true });
            }
            catch (err) {
                this.log.error('TionAuthApi._internalAuthenticate: ', err.message || err.statusCode);
                throw err;
            }
        });
    }
}
exports.TionAuthApi = TionAuthApi;
TionAuthApi.oAuthUrl = 'https://api2.magicair.tion.ru/idsrv/oauth2/token';
TionAuthApi.clientId = 'a750d720-e146-47b0-b414-35e3b1dd7862';
TionAuthApi.clientSecret = 'DTT2jJnY3k2H2GyZ';
class TionFilesystemAuthStorage {
    constructor(log, basePath) {
        this.log = log;
        this.basePath = basePath;
    }
    save(authData) {
        return __awaiter(this, void 0, void 0, function* () {
            fs_1.default.writeFileSync(this.getStatePath(), JSON.stringify(authData, null, 4), { encoding: 'utf8' });
            this.log.info('Auth tokens persisted');
        });
    }
    load() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const text = fs_1.default.readFileSync(this.getStatePath(), { encoding: 'utf8' });
                const ret = JSON.parse(text);
                this.log.info('Got persisted auth tokens');
                return ret;
            }
            catch (err) {
                this.log.info('Auth tokens not persisted');
                throw err;
            }
        });
    }
    getStatePath() {
        return path_1.default.join(this.basePath, 'homebridge-tion.auth.json');
    }
}
exports.TionFilesystemAuthStorage = TionFilesystemAuthStorage;
//# sourceMappingURL=auth.js.map