"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const framework_1 = require("../../homebridge/framework");
const base_1 = require("./base");
const state_1 = require("../state");
const platform_config_1 = require("../../platform_config");
const api_1 = require("../api");
class TionBreezer extends base_1.TionDeviceBase {
    constructor(device, log, config, api, serviceRegistry, characteristicRegistry) {
        super(device, log, config, api, serviceRegistry, characteristicRegistry);
        this.isHeaterInstalled = device.data.heater_installed || false;
        this.maxSpeed = device.max_speed || 0;
        this.maxTargetTemperature = device.t_max || 0;
    }
    addEventHandlers(accessory) {
        accessory.reachable = true;
        if (this.accessories.find(a => a.UUID === accessory.UUID)) {
            return;
        }
        this.accessories.push(accessory);
        const airPurifier = accessory.getService(this.serviceRegistry.AirPurifier);
        const filter = accessory.getService(this.serviceRegistry.FilterMaintenance);
        const heater = accessory.getService(this.serviceRegistry.HeaterCooler);
        if (airPurifier) {
            airPurifier
                .getCharacteristic(this.characteristicRegistry.Active)
                .on('get', callback => this.getState(callback, () => (this.isOn ? 1 : 0)))
                .on('set', (value, callback) => __awaiter(this, void 0, void 0, function* () {
                try {
                    if (!this.isOnline) {
                        return callback('Not reachable');
                    }
                    value = Boolean(value);
                    if (value !== this.isOn) {
                        yield this.setState({
                            is_on: value,
                            speed: this.currentSpeed,
                            speed_min_set: 0,
                            speed_max_set: Math.min(this.speedLimit, this.maxSpeed),
                            heater_enabled: this.isHeaterOn,
                            t_set: this.targetTemperature,
                            gate: this.airIntake,
                        });
                    }
                    this.isOn = value;
                    airPurifier.setCharacteristic(this.characteristicRegistry.CurrentAirPurifierState, value ? 2 : 0);
                    heater.updateCharacteristic(this.characteristicRegistry.Active, value ? this.isHeaterOn : false);
                    callback();
                }
                catch (err) {
                    callback(err.message || err);
                }
            }));
            airPurifier
                .getCharacteristic(this.characteristicRegistry.CurrentAirPurifierState)
                .on('get', callback => this.getState(callback, () => (this.isOn ? 2 : 0)));
            airPurifier
                .getCharacteristic(this.characteristicRegistry.TargetAirPurifierState)
                .setProps({
                minValue: 0,
                maxValue: 1,
                validValues: [1],
            })
                .on('get', callback => this.getState(callback, () => 1));
            airPurifier
                .getCharacteristic(this.characteristicRegistry.RotationSpeed)
                .setProps({
                minValue: 0,
                maxValue: Math.min(this.maxSpeed),
            })
                .on('get', callback => this.getState(callback, () => this.currentSpeed))
                .on('set', (value, callback) => __awaiter(this, void 0, void 0, function* () {
                try {
                    if (!this.isOnline) {
                        return callback('Not reachable');
                    }
                    if (value && value !== this.currentSpeed) {
                        yield this.setState({ speed: value });
                    }
                    this.currentSpeed = value;
                    callback();
                }
                catch (err) {
                    callback(err.message || err);
                }
            }));
        }
        if (filter) {
            filter
                .getCharacteristic(this.characteristicRegistry.FilterChangeIndication)
                .on('get', callback => this.getState(callback, () => this.filterChangeIndication));
            filter
                .getCharacteristic(this.characteristicRegistry.FilterLifeLevel, 0)
                .on('get', callback => this.getState(callback, () => this.filterLifeLevel));
        }
        if (heater) {
            heater
                .getCharacteristic(this.characteristicRegistry.Active)
                .on('get', callback => this.getState(callback, () => (this.isOn && this.isHeaterOn ? 1 : 0)))
                .on('set', (value, callback) => __awaiter(this, void 0, void 0, function* () {
                try {
                    if (!this.isOnline) {
                        return callback('Not reachable');
                    }
                    if (!this.isOn) {
                        return callback();
                    }
                    value = Boolean(value);
                    if (value !== this.isHeaterOn) {
                        yield this.setState({
                            is_on: true,
                            speed: this.currentSpeed,
                            speed_min_set: 0,
                            speed_max_set: Math.min(this.speedLimit, this.maxSpeed),
                            heater_enabled: value,
                            t_set: this.targetTemperature,
                            gate: this.airIntake,
                        });
                    }
                    this.isHeaterOn = Boolean(value);
                    callback();
                }
                catch (err) {
                    callback(err.message || err);
                }
            }));
            heater
                .getCharacteristic(this.characteristicRegistry.CurrentHeaterCoolerState)
                .on('get', callback => this.getState(callback, () => (this.isOn && this.isHeaterOn ? 2 : 0)));
            heater
                .getCharacteristic(this.characteristicRegistry.TargetHeaterCoolerState)
                .setProps({
                maxValue: 2,
                minValue: 0,
                validValues: [1],
            })
                .on('get', callback => this.getState(callback, () => 1));
            heater
                .getCharacteristic(this.characteristicRegistry.CurrentTemperature)
                .on('get', callback => this.getState(callback, () => this.currentTemperature));
            heater
                .getCharacteristic(this.characteristicRegistry.HeatingThresholdTemperature)
                .setProps({
                minValue: 0,
                maxValue: this.maxTargetTemperature,
                minStep: 1,
            })
                .on('get', callback => this.getState(callback, () => this.targetTemperature))
                .on('set', (value, callback) => __awaiter(this, void 0, void 0, function* () {
                try {
                    if (!this.isOnline) {
                        return callback('Not reachable');
                    }
                    if (!this.isOn) {
                        return callback();
                    }
                    if (value !== this.targetTemperature) {
                        yield this.setState({
                            is_on: true,
                            speed: this.currentSpeed,
                            speed_min_set: 0,
                            speed_max_set: Math.min(this.speedLimit, this.maxSpeed),
                            heater_enabled: true,
                            t_set: value,
                            gate: this.airIntake,
                        });
                    }
                    this.targetTemperature = value;
                    callback();
                }
                catch (err) {
                    callback(err.message || err);
                }
            }));
        }
    }
    updateState(state) {
        const action = this.isCommandRunning ? 'updateCharacteristic' : 'setCharacteristic';
        this.parseState(state);
        this.accessories.forEach(accessory => {
            accessory.reachable = this.isOnline;
            const airPurifier = accessory.getService(this.serviceRegistry.AirPurifier);
            if (airPurifier) {
                airPurifier[action](this.characteristicRegistry.Active, this.isOn ? 1 : 0);
                airPurifier[action](this.characteristicRegistry.CurrentAirPurifierState, this.isOn ? 2 : 0);
                airPurifier[action](this.characteristicRegistry.TargetAirPurifierState, 1);
                airPurifier[action](this.characteristicRegistry.RotationSpeed, this.currentSpeed);
            }
            const filter = accessory.getService(this.serviceRegistry.FilterMaintenance);
            if (filter) {
                filter[action](this.characteristicRegistry.FilterChangeIndication, this.filterChangeIndication ? 1 : 0);
                filter[action](this.characteristicRegistry.FilterLifeLevel, this.filterLifeLevel);
            }
            const heater = accessory.getService(this.serviceRegistry.HeaterCooler);
            if (heater) {
                heater[action](this.characteristicRegistry.Active, this.isOn && this.isHeaterOn ? 1 : 0);
                heater[action](this.characteristicRegistry.CurrentHeaterCoolerState, this.isOn && this.isHeaterOn ? 2 : 0);
                heater[action](this.characteristicRegistry.TargetHeaterCoolerState, 1);
                heater[action](this.characteristicRegistry.CurrentTemperature, this.currentTemperature);
                heater[action](this.characteristicRegistry.HeatingThresholdTemperature, this.targetTemperature);
            }
        });
    }
    parseState(state) {
        const device = this.findDeviceInState(state);
        if (!device) {
            return false;
        }
        this.isOnline = device.is_online;
        this.isOn = device.data.is_on || false;
        this.currentSpeed = device.data.speed || 0;
        this.speedLimit = device.data.speed_limit || 0;
        if (this.isHeaterInstalled) {
            this.isHeaterOn = device.data.heater_enabled || false;
            this.targetTemperature = device.data.t_set || 0;
            this.outsideTemperature = device.data.t_in || 0;
            this.currentTemperature = device.data.t_out || 0;
        }
        this.filterChangeIndication = device.data.filter_need_replace || false;
        this.filterLifeLevel = device.data.filter_time_seconds
            ? device.data.filter_time_seconds / (device.data.filter_time_seconds + (device.data.run_seconds || 0))
            : 0;
        this.airIntake = device.data.gate;
        return true;
    }
}
exports.TionBreezer = TionBreezer;
//# sourceMappingURL=breezer.js.map